SystemOrganization addCategory: #'WebClient-Seaside'!

WAServerAdaptor subclass: #WAWebServerAdaptor
	instanceVariableNames: 'server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebClient-Seaside'!

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 7/20/2010 21:02'!
basicStart
	"Start the WebServer instance"

	server := WebServer new.
	server addService: '/' action: (MessageSend receiver: self selector: #process:).
	server listenOn: self port.
! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 7/20/2010 21:15'!
basicStop
	"Stop the WebServer instance if it is running"

	server ifNotNil:[
		server destroy.
		server := nil.
	].
! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 7/20/2010 21:15'!
isRunning
	"Returns true if the WebServer instance is currently running"

	^server notNil and:[server isRunning]! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 7/20/2010 21:15'!
isStarting
	"Returns true if the WebServer instance is currently starting"

	^false! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 7/20/2010 21:16'!
isStopped
	"Returns true if the WebServer instance is currently stopped"

	^server isNil or:[server isRunning not]! !

!WAWebServerAdaptor methodsFor: 'testing' stamp: 'ar 7/20/2010 21:16'!
isStopping
	"Returns true if the WebServer instance is currently stopping"

	^false! !

!WAWebServerAdaptor methodsFor: 'processing' stamp: 'ar 5/4/2010 20:44'!
process: webRequest
	"Process the request; send the response back to the client"

	| webResponse |
	webResponse := super process: webRequest.
	webRequest sendResponse: webResponse content: webResponse content.
! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'pmm 6/19/2010 07:06'!
requestAddressFor: webRequest
	"Answer the remote socket address as a string"

	| first |
	first := true.
	^ String streamContents: [ :stream |
		webRequest stream socket remoteAddress do: [ :each |
			first
				ifTrue: [ first := false ]
				ifFalse: [ stream nextPut: $. ].
			stream nextPutAll: each asString ] ]! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:54'!
requestBodyFor: webRequest
	"Answer the undecoded request body as a String."

	^(webRequest contentLength = nil and:[webRequest isGetRequest])
		ifTrue:[nil]
		ifFalse:[webRequest content]! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 7/20/2010 21:06'!
requestCookiesFor: webRequest
	"Answer a collection of WARequestCookies from the request"

	^Array streamContents:[:s|
		webRequest headersAt: 'cookie' do:[:hdr|
			s nextPut: (WARequestCookie fromString: hdr)]]! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 7/20/2010 20:34'!
requestFieldsFor: webRequest
	"Convert the post fields of the arguments into WARequestFields"

	| fields |
	fields := WARequestFields new.
	webRequest isPostRequest ifFalse:[^fields].
	webRequest contentType caseOf: {
		['application/x-www-form-urlencoded'] -> [
			WebUtils 
				decodeUrlEncodedForm: webRequest content 
				do:[:key :value| fields at: key add: (self codec url decode: value)].
		].
		['multipart/form-data'] -> [
			WebUtils 
				decodeMultipartForm: webRequest contentStream 
				boundary: webRequest multiparyBoundary 
				do:[:headers :params :content| 
					| key value filename mimetype |
					key := params at: 'name'.
					filename := params at: 'filename' ifAbsent:[nil].
					"Only used for file uploads so it's okay to default to 
					application/octet-stream even though rfc 2388 says 
					the default should be text/plain (for non-uploads)"
					mimetype := headers at: 'Content-Type' 
									ifAbsent:['application/octet-stream'].
					value := filename
						ifNil:[self codec url decode: content]
						ifNotNil:[
							(WAFile new)
								fileName: (self codec url decode: filename);
								contentType: (WAMimeType fromString: mimetype);
								contents: content asByteArray;
							yourself.
						].
					fields at: key add: value.
				].
		].
	} otherwise:[].
	^ fields! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 7/20/2010 21:13'!
requestHeadersFor: webRequest
	"Answer the request headers for the webRequest."

	| dict |
	dict := Dictionary new.
	webRequest headersDo:[:key :val|
		"RFC 2616: Multiple message-header fields with the same field-name MAY be present in a message if and only if the entire field-value for that header field is defined as a comma-separated list [i.e., #(values)]. It MUST be possible to combine the multiple header fields into one 'field-name: field-value' pair, without changing the semantics of the message, by appending each subsequent field-value to the first, each separated by a comma."
		(dict includesKey: key)
			ifTrue:[dict at: key modify:[:old| old, ', ', val]]
			ifFalse:[dict at: key put: val]].
	^dict! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:45'!
requestMethodFor: webRequest
	"Answers the request method (GET, POST etc)"

	^webRequest method! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:46'!
requestUrlFor: webRequest
	"Answers a WAUrl from the given request"

	^(WAUrl absolute: webRequest rawUrl)
		decodedWith: self codec! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 16:46'!
requestVersionFor: webRequest
	"Answers the WAHttpVersion from the request protocol"

	^ WAHttpVersion fromString: webRequest protocol! !

!WAWebServerAdaptor methodsFor: 'converting' stamp: 'ar 5/4/2010 20:44'!
responseFrom: aRequestContext
	"Answer a native response from aRequestContext."

	| response |
	response := WebResponse new.
	response protocol: aRequestContext request version asString 
			code: aRequestContext response status.
	aRequestContext response headers keysAndValuesDo:[ :key :value | 
		response headerAt: key put: value greaseString.
	].
	aRequestContext response cookies do:[:cookie|
		response headerAt: 'set-cookie' put: cookie oldNetscapeString.
	].
	response content: aRequestContext response stream contents.
	^response! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 7/20/2010 21:15'!
shutDown
	"The system is shutting down. Perform necessary cleanup."
! !

!WAWebServerAdaptor methodsFor: 'private' stamp: 'ar 7/20/2010 21:15'!
startUp
	"The system is starting up. Perform necessary initialization."
! !
